import { rootAPI } from '../api/APIHelper'

const fmuEndpoint = rootAPI + '/fmus/'
export class DigitalTwin {
  /*
      This is class retrieves all the necessary information needed for the USG.js module to create a 3D visualisation
      of the model.

      Two types of files are needed:
          - JSON Master file
              - List of parts
              - Base ID for each part
              - Local to global coordinate system transformation matrix for each part
          - JSON Part file (For each part in model)
              - Part geometry description
   */

  constructor (name, myApp) {
    // Initiate variables
    this.name = name
    this.fileName = this.name.concat('.json')
    this.directory = fmuEndpoint.concat(this.name) // /js/Name/
    this.parts = []
    this.arrays = []
    this.baseId = []
    this.myApp = myApp
    // Create Model
    this.createModel()
  }

  // Return part index
  getPartIndex (baseID) {
    /*
        To properly select a part when updating its position we need the index it has been given. The index of a
        part corresponds to the parts position in the list of parts.
        e.g:
        this.parts = [arm, base, pulley, upperArm, ....] thus the index of "pulley" is 2.
     */

    let index = -1

    for (let i = 0; i < this.baseId.length; i++) {
      if (this.baseId[i] === baseID) {
        index = i
      }
    }
    return index
  }

  // Return array with Base IDs
  getIDS () {
    return this.baseId
  }

  // Create 3D model
  async createModel () {
    // Retrieve information from JSON master file
    await this.findParts()

    // Generate 3D visualisation for each part
    for (let i = 0; i < this.parts.length; i++) {
      await this.loadParts(this.parts[i], this.arrays[i], i)
    }
  }

  async loadParts (Name, Arr, index) {
    /*
        This function loads part geometry and calls the function "addPartGeometry" which uses the Ceetron USG module
        to create the 3D visualisation of this part.
     */

    const partEndPoint = this.directory.concat('/models/').concat(Name)
    try {
      const response = await fetch(partEndPoint, { cors: 'no-cors' })
      const data = await response.json()
      this.myApp.addPartGeometry(data, Arr[0], Arr[1], Arr[2], index)
    } catch (error) {
      console.log(error)
    }
  }

  async findParts () {
    /*
         Retrieve information from JSON master file:
            - List of parts
            - Base ID for each part
            - Local to global coordinate system transformation matrix for each part
     */
    const masterFileEndpoint = this.directory.concat('/models/').concat(this.fileName)
    try {
      const response = await fetch(masterFileEndpoint, { cors: 'no-cors' })
      const data = await response.json()
      let files = data.ListOfFile
      let coordinates = data.Coordinates
      let IDs = data.baseID

      for (let i = 0; i < files.length; i++) {
        const index = i * 3
        files[i] = files[i].replace('.ftl', '.json')
        this.parts.push(files[i])
        this.baseId.push(IDs[i])
        this.arrays.push([coordinates[index], coordinates[index + 1], coordinates[index + 2]])
      }
    } catch (error) {
      console.log(error)
    }
  }
}
