"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var cee = require("../ceetron/CeeCloudClientComponent");
// Initialiser for Ceetron module of application
function startApp(canvasElement) {
    return new App(canvasElement);
}
exports.startApp = startApp;
// Class containing Ceetron Cloud Client Component state
var App = /** @class */ (function () {
    function App(canvas) {
        var _this = this;
        this.canvas = canvas;
        // Initialise Ceetron Cloud Client Component
        this.cloudSession = new cee.CloudSession();
        var viewer = this.cloudSession.addViewer(canvas);
        if (!viewer) {
            throw ("No WebGL support");
        }
        this.view = viewer.addView();
        this.model = new cee.usg.UnstructGridModel();
        this.view.addModel(this.model);
        this.state = this.model.addState();
        this.state.geometry = new cee.usg.Geometry();
        // Hide infoBox initially
        this.view.overlay.infoBoxVisible = false;
        // Listen for resize events
        window.addEventListener('resize', function () { return _this._handleWindowResizeEvent(); });
        // Manually run resize function once
        this._handleWindowResizeEvent();
        // Update view every browser frame
        window.requestAnimationFrame(function (time) { return _this._myAnimationFrameCallback(time); });
    }
    // Adjust view dimension (called when window is resized)
    App.prototype._handleWindowResizeEvent = function () {
        var canvasWidth = window.innerWidth;
        var canvasHeight;
        // @ts-ignore
        canvasHeight = this.canvas.parentElement.offsetHeight;
        this.cloudSession.getViewerAt(0).resizeViewer(canvasWidth, canvasHeight);
    };
    // Force the layout to resize, same reason as in plotcomponent
    App.prototype.resizeLayout = function () {
        var canvasHeight;
        // @ts-ignore
        canvasHeight = this.canvas.parentElement.offsetHeight;
        var canvasWidth;
        // @ts-ignore
        canvasWidth = this.canvas.parentElement.offsetWidth;
        this.cloudSession.getViewerAt(0).resizeViewer(canvasWidth, canvasHeight);
    };
    // Update view (called every browser frame)
    App.prototype._myAnimationFrameCallback = function (highResTimestamp) {
        var _this = this;
        this.cloudSession.handleAnimationFrameCallback(highResTimestamp);
        window.requestAnimationFrame(function (time) { return _this._myAnimationFrameCallback(time); });
    };
    // Create the torsion rod geometry
    App.prototype.addRodGeometry = function (data) {
        var geometry = this.state.geometry.addPart();
        geometry.mesh = new cee.usg.Mesh(data.nodeArr, data.elementTypeArr, data.elementNodeIndexArr);
        geometry.settings.color = new cee.Color3(.8, .8, .8);
        // Transform to global coordinate system
        var c = cee.Mat4.fromElements(1, 0, 0, -0.02407066, 0, 1, 0, -0.02722985, 0, 0, 1, 0.27199998, 0, 0, 0, 1);
        this.state.setPartTransformationAt(1, c);
    };
    App.prototype.addPartGeometry = function (data, a1, a2, a3, index) {
        var geometry = this.state.geometry.addPart();
        geometry.mesh = new cee.usg.Mesh(data.nodeArr, data.elementTypeArr, data.elementNodeIndexArr);
        geometry.settings.color = new cee.Color3(Math.random(), Math.random(), Math.random());
        var c = cee.Mat4.fromElements(a1[0], a1[1], a1[2], a1[3], a2[0], a2[1], a2[2], a2[3], a3[0], a3[1], a3[2], a3[3], 0, 0, 0, 1);
        this.state.setPartTransformationAt(index, c);
        //this.showStatistics(this.state.geometry);
    };
    App.prototype.showStatistics = function (geometry) {
        // Generate statistics on geometry
        var nodeCount = 0;
        var elementCount = 0;
        for (var _i = 0, _a = geometry.getPartArray(); _i < _a.length; _i++) {
            var part = _a[_i];
            nodeCount += part.mesh.nodeCount;
            elementCount += part.mesh.elementCount;
        }
        // Log generated statistics
        console.log("Initial state loaded, nodeCount=" + nodeCount + ", elementCount=" + elementCount);
        // Draw generated statistics in bottom right corner
        this.view.overlay.infoBoxVisible = true;
        this.view.overlay.setInfoBoxContent("Elements: " + elementCount + " elements \nNodes: " + nodeCount + " nodes");
    };
    App.prototype.updateDisplacement = function (transformationMatrix, baseID) {
        // Create Ceetron matrix from transformation data
        var m = cee.Mat4.fromArray(transformationMatrix);
        var localToGlobalTransformation = cee.Mat4.fromElements(1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1);
        var transformation = cee.Mat4.multiply(m, localToGlobalTransformation);
        // Apply transformation to armGeometry
        this.state.setPartTransformationAt(baseID, transformation);
    };
    // Change drawing style for geometries
    App.prototype.setDrawStyle = function (ds) {
        var geometry = this.model.getStateAt(0).geometry;
        for (var _i = 0, _a = geometry.getPartArray(); _i < _a.length; _i++) {
            var part = _a[_i];
            if (ds === "surface")
                part.settings.drawStyle = cee.usg.DrawStyle.SURFACE;
            else if (ds === "surface_mesh")
                part.settings.drawStyle = cee.usg.DrawStyle.SURFACE_MESH;
            else if (ds === "outline_mesh")
                part.settings.drawStyle = cee.usg.DrawStyle.SURFACE_OUTLINE_MESH;
            else if (ds === "lines")
                part.settings.drawStyle = cee.usg.DrawStyle.LINES;
            else if (ds === "points")
                part.settings.drawStyle = cee.usg.DrawStyle.POINTS;
            else if (ds === "outline")
                part.settings.drawStyle = cee.usg.DrawStyle.OUTLINE;
        }
    };
    return App;
}());
exports.App = App;
