import * as cee from "../ceetron/CeeCloudClientComponent";

// Initialiser for Ceetron module of application
export function startApp(canvasElement: HTMLCanvasElement): App {
    return new App(canvasElement);
}

// Class containing Ceetron Cloud Client Component state
export class App {

    // Ceetron Cloud Client Component state
    private cloudSession: cee.CloudSession;
    private view: cee.View;
    private model: cee.usg.UnstructGridModel;
    private state: cee.usg.State;

    // Canvas containing visualisation
    private canvas: HTMLCanvasElement;

    constructor(canvas: HTMLCanvasElement) {
        this.canvas = canvas;

        // Initialise Ceetron Cloud Client Component
        this.cloudSession = new cee.CloudSession();
        let viewer = this.cloudSession.addViewer(canvas);
        if (!viewer) {
            throw("No WebGL support");
        }
        this.view = viewer.addView();
        this.model = new cee.usg.UnstructGridModel();
        this.view.addModel(this.model);
        this.state = this.model.addState();
        this.state.geometry = new cee.usg.Geometry();

        // Hide infoBox initially
        this.view.overlay.infoBoxVisible = false;

        // Listen for resize events
        window.addEventListener('resize', () => this._handleWindowResizeEvent());

        // Manually run resize function once
        this._handleWindowResizeEvent();

        // Update view every browser frame
        window.requestAnimationFrame((time: number) => this._myAnimationFrameCallback(time));
    }

    // Adjust view dimension (called when window is resized)
    private _handleWindowResizeEvent() {
        let canvasWidth = window.innerWidth;
        let canvasHeight: number;
        // @ts-ignore
        canvasHeight = this.canvas.parentElement.offsetHeight;
        this.cloudSession.getViewerAt(0).resizeViewer(canvasWidth, canvasHeight);
    }

    // Force the layout to resize, same reason as in plotcomponent
    public resizeLayout() {
        let canvasHeight: number;
        // @ts-ignore
        canvasHeight = this.canvas.parentElement.offsetHeight;
        let canvasWidth: number;
        // @ts-ignore
        canvasWidth = this.canvas.parentElement.offsetWidth;
        this.cloudSession.getViewerAt(0).resizeViewer(canvasWidth, canvasHeight);
    }

    // Update view (called every browser frame)
    private _myAnimationFrameCallback(highResTimestamp:number) {
        this.cloudSession.handleAnimationFrameCallback(highResTimestamp);
        window.requestAnimationFrame((time: number) => this._myAnimationFrameCallback(time));
    }





    // Create the torsion rod geometry
    addRodGeometry(data: any) {
        let geometry = this.state.geometry.addPart();
        geometry.mesh = new cee.usg.Mesh(data.nodeArr, data.elementTypeArr, data.elementNodeIndexArr);
        geometry.settings.color = new cee.Color3(.8, .8, .8);

        // Transform to global coordinate system
        const c = cee.Mat4.fromElements(
            1, 0, 0, -0.02407066,
            0, 1, 0, -0.02722985,
            0, 0, 1, 0.27199998,
            0, 0, 0, 1
        );
        this.state.setPartTransformationAt(1, c);
    }

    addPartGeometry(data: any,a1: any,a2: any,a3: any,index: any){
        let geometry = this.state.geometry.addPart();
        geometry.mesh = new cee.usg.Mesh(data.nodeArr, data.elementTypeArr, data.elementNodeIndexArr);
        geometry.settings.color = new cee.Color3(Math.random(),Math.random(),Math.random());


        const c = cee.Mat4.fromElements(
            a1[0], a1[1], a1[2], a1[3],
            a2[0], a2[1], a2[2], a2[3],
            a3[0], a3[1], a3[2], a3[3],
            0, 0, 0, 1
        );

        this.state.setPartTransformationAt(index, c);
        //this.showStatistics(this.state.geometry);
    }

    private showStatistics(geometry: any) {
        // Generate statistics on geometry

        let nodeCount = 0;
        let elementCount = 0;
        for (let part of geometry.getPartArray()) {
            nodeCount += part.mesh.nodeCount;
            elementCount += part.mesh.elementCount;
        }

        // Log generated statistics
        console.log("Initial state loaded, nodeCount=" + nodeCount + ", elementCount=" + elementCount);

        // Draw generated statistics in bottom right corner
        this.view.overlay.infoBoxVisible = true;
        this.view.overlay.setInfoBoxContent(`Elements: ${elementCount} elements \nNodes: ${nodeCount} nodes`);
    }

    updateDisplacement(transformationMatrix: number[],baseID: any) {
        // Create Ceetron matrix from transformation data
        const m = cee.Mat4.fromArray(transformationMatrix);
        const localToGlobalTransformation = cee.Mat4.fromElements(
            1, 0, 0, 0,
            0, 1, 0, 0,
            0, 0, 1, 0,
            0, 0, 0, 1
        );
        const transformation = cee.Mat4.multiply(m,localToGlobalTransformation);

        // Apply transformation to armGeometry
        this.state.setPartTransformationAt(baseID, transformation);
    }

    // Change drawing style for geometries
    setDrawStyle(ds: string) {
        const geometry = this.model.getStateAt(0).geometry;
        for (let part of geometry.getPartArray()) {
            if      (ds === "surface")              part.settings.drawStyle = cee.usg.DrawStyle.SURFACE;
            else if (ds === "surface_mesh")         part.settings.drawStyle = cee.usg.DrawStyle.SURFACE_MESH;
            else if (ds === "outline_mesh")         part.settings.drawStyle = cee.usg.DrawStyle.SURFACE_OUTLINE_MESH;
            else if (ds === "lines")                part.settings.drawStyle = cee.usg.DrawStyle.LINES;
            else if (ds === "points")               part.settings.drawStyle = cee.usg.DrawStyle.POINTS;
            else if (ds === "outline")              part.settings.drawStyle = cee.usg.DrawStyle.OUTLINE;
        }
    }
}

