import { EventBus, EVENTS } from '../js/EventBus'
import { subscribeToSource, unSubscribeSource } from '../api/APIHelper'
import { deepCopy } from '../utils/util'

const struct = require('@aksel/structjs')

let decoder = new TextDecoder('utf-8')

function byteToString (buf) {
  return decoder.decode(buf)
}

export default {
  data () {
    return {
      socketEndpoint: 'ws://129.241.90.187:1337',
      isOpen: false,
      isConnecting: false,
      sourceBuffers: {},
      pushDataIntervalID: undefined,
      packetCounter: 0,
      prevSubscriptions: []
    }
  },
  computed: {
    subscribedSources () {
      return this.$store.state.channelModule.sourceDict
    }
  },
  watch: {
    isOpen (isOpen) {
      isOpen ? this.initWebSocketConnection() : this.closeWebSocketConnection()
    }
  },
  created () {
    EventBus.$on(EVENTS.subscribe, this.subscribeDataSources)
  },
  beforeDestroy () {
    EventBus.$off(EVENTS.subscribe, this.subscribeDataSources)
  },
  methods: {
    async subscribeDataSources (sources) {
      const removedSources = this.prevSubscriptions.filter((sub) => !sources.some((source) => source.id === sub.id))
      const addedSources = sources.filter(
        (source) => !this.prevSubscriptions.some((prevSub) => prevSub.id === source.id)
      )
      removedSources.forEach((source) => {
        unSubscribeSource(source.id)
      })
      addedSources.forEach((source) => {
        subscribeToSource(source.id)
      })
      this.prevSubscriptions = sources.map((source) => ({
        id: source.id,
        url: source.url
      }))
      if (!this.isOpen) {
        await this.fetchAuthCookie()
        await this.initWebSocketConnection()
      } else {
        this.initBuffers()
      }
    },
    async fetchAuthCookie () {
      await fetch('http://129.241.90.187:1337/session', {
        credentials: 'include'
      })
    },
    async initWebSocketConnection () {
      // eslint-disable-next-line no-undef
      if (this.ws) return
      this.ws = new WebSocket(this.socketEndpoint)
      this.isConnecting = true
      this.ws.binaryType = 'arraybuffer'
      this.ws.onopen = () => {
        this.isConnecting = false
        this.isOpen = true
      }
      this.ws.onerror = () => {
        this.isConnecting = false
        this.$root.displayPopup("Couldn't reach server")
        this.isOpen = false
      }
      this.ws.onclose = () => {
        this.isOpen = false
        clearInterval(this.pushDataIntervalID)
      }
      this.initParser()
      this.ws.onmessage = (event) => {
        if (event.data.byteLength > 0) {
          const data = event.data
          const sourceID = byteToString(new Uint8Array(data, 0, 4))
          this.parseData(data.slice(4), sourceID)
        } else {
          console.log('pong')
        }
      }
    },
    closeWebSocketConnection () {
      this.ws.close(1000, 'Deliberate disconnection')
      clearInterval(this.pushDataIntervalID)
      this.ws = null
    },
    parseData (data, sourceID) {
      const sourceBuffer = this.sourceBuffers[sourceID]
      if (sourceBuffer === undefined) {
        return
      }
      this.packetCounter++
      const unpacker = sourceBuffer.unpacker
      const unpackIterator = unpacker.iter_unpack(data)
      let unpacked = unpackIterator.next().value
      while (unpacked) {
        sourceBuffer.x_buffer.push(new Date(unpacked[0] * 1000))
        const channelsIds = this.subscribedSources[sourceID].channels.map((it) => it.id)
        channelsIds.forEach((channelID) => {
          sourceBuffer.y_buffer[channelID].push(unpacked[channelID + 1])
        })
        unpacked = unpackIterator.next().value
      }
    },
    async pushData () {
      if (this.packetCounter > 0) {
        EventBus.$emit(EVENTS.newData, deepCopy(this.sourceBuffers))
        this.resetBuffers()
        this.packetCounter = 0
      }
    },
    initParser () {
      this.initBuffers()
      this.pushDataIntervalID = setInterval(this.pushData, 100)
    },
    resetBuffers () {
      for (const sourceId in this.sourceBuffers) {
        let sourceBuffer = this.sourceBuffers[sourceId]
        sourceBuffer.x_buffer = []
        Object.keys(sourceBuffer.y_buffer).forEach((channelID) => {
          sourceBuffer.y_buffer[channelID] = []
        })
      }
    },
    initBuffers () {
      this.sourceBuffers = {}
      for (const sourceId in this.subscribedSources) {
        let sourceBuffer = {}
        const byteFormat = this.subscribedSources[sourceId].byteFormat
        sourceBuffer.unpacker = struct(byteFormat)
        const channels = this.subscribedSources[sourceId].channels
        sourceBuffer.x_buffer = []
        sourceBuffer.y_buffer = {}
        channels.forEach((channel) => {
          sourceBuffer.y_buffer[channel.id] = []
        })
        this.sourceBuffers[sourceId] = sourceBuffer
      }
    }
  }
}
